<?php

/**
 * @version		mod_scorpion_openinghours_helper.php - 12/11/2024
 * @author		Edwin Witlox $
 * @copyright	Copyright (C) Scorpion Computers and Software, Inc. All rights reserved.
 * @license		GNU General Public License version 2 or later; http://www.gnu.org/copyleft/gpl.html
 * @email		info@ScorpionComputers.nl
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;

class ScorpionOpeninghoursHelper {
	// Setting up private variables to use
	private $moduleNameTech = 'mod_scorpion_openinghours';
    private $moduleNameFull = 'Scorpion Opening Hours';
	private $currentUser;
    private $params;
	private $document;
	private $baseURL;
    private $moduleID;
	private $moreStyle;
	private $chosenTemplate;
	private $openingHoursList;
	private $selectedTimezone;

	// Adding some default values to the variables
	public function __construct($params, $id) {
        $this->params           = $params;
        $this->moduleID         = $id;
		$this->moreStyle		= $this->params->get('moreStyle');
		$this->chosenTemplate	= substr( $this->params->get('layout', 'default'), 2);
		$this->openingHoursList	= array($this->params->get('Mondaytimes1'));
		array_push($this->openingHoursList, $this->params->get('Tuesdaytimes1'));
		array_push($this->openingHoursList, $this->params->get('Wednesdaytimes1'));
		array_push($this->openingHoursList, $this->params->get('Thursdaytimes1'));
		array_push($this->openingHoursList, $this->params->get('Fridaytimes1'));
		array_push($this->openingHoursList, $this->params->get('Saturdaytimes1'));
		array_push($this->openingHoursList, $this->params->get('Sundaytimes1'));
		$this->selectedTimezone	= $this->getUsersOrSelectedTimezone();
		
		// Setting default variables to use in the module
		$this->$moduleID = 'scs-opening-hours-'.$id;
		$this->document = Factory::getDocument();
		$this->baseURL = Uri::root(true);
	}

	public function getFieldValue($fieldName, $defaultValue) {
		return $this->params->get($fieldName, $defaultValue);
	}

	// Add the basic CSS file from this module
	public function addBasicCSS() {
		$urlCSS = $this->baseURL.'/modules/mod_scorpion_openinghours/mod_scorpion_openinghours.css';
		$this->document->addStyleSheet($urlCSS);
		// Add the specific CSS file for this template
		$templateCSS = $this->baseURL.'/modules/mod_scorpion_openinghours/tmpl/'.$this->chosenTemplate.'.css';
		$this->document->addStylesheet($templateCSS);
	}

	// Getting the parameter values for use in the rest of the module
	public function addExtraCSS(&$extraStyle) 
	{
		// Adding style tag to the top of the HTML document for better support by browsers
		$stylelink = '<style>'."\r\n";
		$stylelink .= '/* Styling for '.$this->moduleNameFull.' */'."\r\n";
		$stylelink .= $extraStyle."\r\n";
		$stylelink .= $this->moreStyle."\r\n";
		$stylelink .= '</style>';
		$this->document->addCustomTag($stylelink);
	}

	// Adding the basic HTML code for the start and end of the module
	public function addBasicHTML($StartOrEnd, $templateName = null) {
		$htmlCode = '';
		if($StartOrEnd == 'start') {
			$htmlCode .= "\r\n".'<!-- '.$this->moduleNameFull.' - BEGIN -->'."\r\n";
			$htmlCode .= '<div id='.$this->$moduleID.' class="scs-opening-hours" data-template="'.$templateName.'">'."\r\n";
		} elseif ($StartOrEnd == 'end') {
			$htmlCode .= '</div>'."\r\n";
			$htmlCode .='<!-- '.$this->moduleNameFull.' - END -->'."\r\n";
		}
		echo $htmlCode;
	}

	// Adding the basic HTML code for the additional text at the start and/or end of the openinghours
	public function getAdditionalTextHTML($BeforeOrAfter) {
		$htmlCode = '';
		if($BeforeOrAfter == 'before') {
			if($this->params->get('useTextBefore')) {
				$htmlCode .= ''."\r\n";
				$htmlCode .= '<div>'."\r\n";
				$htmlCode .= $this->params->get('textBefore')."\r\n";
				$htmlCode .= '</div>'."\r\n";
			}
		} elseif ($BeforeOrAfter == 'after') {
			if($this->params->get('useTextAfter')) {
				$htmlCode .= ''."\r\n";
				$htmlCode .= '<div>'."\r\n";
				$htmlCode .= $this->params->get('textAfter')."\r\n";
				$htmlCode .= '</div>'."\r\n";
			}
		}
		return $htmlCode;
	}

	// If a user is logged in and has a specific timezone set on the website we can use that one 
	// instead of the selected in the parameters of the module
	public function getUsersOrSelectedTimezone() {
		// See if a user is logged in
		$this->$currentUser = Factory::getApplication()->getIdentity();
		if($this->$currentUser->id <> 0){
			// If a user is logged in get it's selected timezone
			$UsersTimezone = $this->$currentUser->getParam('timezone');
			if($UsersTimezone) {
				// If the user has set a specific timezone, we use that instead of the timezone selected in in the module
				return $UsersTimezone;
			} else {
				return $this->params->get('Timezone');
			}
		} else {
			return $this->params->get('Timezone');
		}
	}
	
	// Function to return a class-name if given daynumber is current day
	public function getCurrentDayOpeningtimes($dayNumber) {
		return $this->openingHoursList[$dayNumber-1];
	}

	// Function to return if given daynumber is current day
	public function isThisCurrentDay($dayNumber) {
		$currentDayNr = Factory::getDate()->format('N');
		if ($dayNumber == $currentDayNr) {
			return true;
		} else {
			return false;
		}
	}

	// Function to return the openinghours of a given daynumber
	public function getCurrentDayClass($dayNumber) {
		$result = '';
		$currentDayNr = Factory::getDate()->format('N');
		if ($dayNumber == $currentDayNr) {
			if($this->params->get('HighlightToday',0)) {
				$result .= ' scs-current-day';
			}
			if ($this->areWeOpenNow()) {
				$result .= ' scs-open';
			} else {
				$result .= ' scs-closed';
			}
		}
		return $result;
	}

	// Function to determine if we are open or closed
	public function areWeOpenNow() {
		$userTimezone = $this->getUsersOrSelectedTimezone();
		$currentTime = new DateTime('now', new DateTimeZone($userTimezone));
		$currentDate = clone $currentTime;
		$currentDate->setTime(0, 0, 0);
		
		// First check if there are any additional opening hours for today
		$additionalHours = $this->getAdditionalOpeningHours();
		foreach ($additionalHours as $hour) {
			$dateFrom = clone $hour['date_from'];
			$dateTo = clone $hour['date_to'];
			$dateFrom->setTime(0, 0, 0);
			$dateTo->setTime(23, 59, 59);
			
			// Check if today falls within the additional opening hours date range
			if ($currentDate >= $dateFrom && $currentDate <= $dateTo) {
				if ($hour['open_or_closed'] === 'closed') {
					// Additional hours say we're closed today
					return false;
				} else {
					// Additional hours say we're open today
					if ($hour['fullday'] === '1') {
						// Full day open
						return true;
					} else {
						// Check specific times
						$startTime = $hour['start_time'];
						$endTime = $hour['end_time'];
						$currentTimeString = $currentTime->format('H:i');
						
						if (!empty($startTime) && !empty($endTime)) {
							if (strtotime($currentTimeString) >= strtotime($startTime) && strtotime($currentTimeString) <= strtotime($endTime)) {
								return true;
							} else {
								return false;
							}
						}
					}
				}
			}
		}
		
		// No additional hours for today, check regular opening hours
		$result = false;
		$currentDayNr = Factory::getDate()->format('N');
		// retrieve the opening hours of today
		$currentOpeningTime = str_replace(' ','',$this->openingHoursList[$currentDayNr-1]);
		
		// Check if it's marked as closed
		if (strtoupper($currentOpeningTime) === 'CLOSED' || strtoupper($currentOpeningTime) === 'GESLOTEN') {
			return false;
		}
		
		// and split them into usable timeslots
		$times = explode('-',$currentOpeningTime,99);
		if (count($times) >= 2) {
			// only get the time to compare without the date, otherwise the comparisment will go wrong
			$currentTimeString = $currentTime->format('H:i');
			if(strtotime($currentTimeString) >= strtotime($times[0]) && strtotime($currentTimeString) <= strtotime($times[1]) ) {
				$result = true;
			}
		}
		
		return $result;
	}

	// function to retrieve the full text for open or closed
	public function getOpenClosedText() {
		$result = '';
		if( strlen($this->params->get('PreOpenClosedText')) > 0 ) {
			$result .= '<span class="scs-pre-open-closed">';
			$result .= $this->params->get('PreOpenClosedText').'&nbsp;';
			$result .= '</span>';
		}
		if( $this->areWeOpenNow() ) {
			$result .= '<span class="scs-open-closed scs-open">';
			$result .= $this->params->get('OpenText');
		} else {
			$result .= '<span class="scs-open-closed scs-closed">';
			$result .= $this->params->get('ClosedText');
		}
		$result .= '</span>';
		return $result;
	}

	// function to just add a class if open or closed
	public function getOpenClosedClass() {
		$result = '';
		if( $this->areWeOpenNow() ) {
			$result .= 'scs-open';
		} else {
			$result .= 'scs-closed';
		}
		return $result;
	}

	// Function to get additional opening hours
	public function getAdditionalOpeningHours() {
		$additionalHours = $this->params->get('additional_opening_hours', array());
		$result = array();
		
		// Convert stdClass to array if needed
		if (is_object($additionalHours)) {
			$additionalHours = json_decode(json_encode($additionalHours), true);
		}
		
		if (!empty($additionalHours)) {
			$userTimezone = $this->getUsersOrSelectedTimezone();
			$currentDate = new DateTime('now', new DateTimeZone($userTimezone));
			// Set time to start of day for proper date comparison
			$currentDate->setTime(0, 0, 0);
			
			$showFuture = $this->params->get('show_future_additionalhours', 1);
			$numberToShow = $this->params->get('number_future_additionalhours', 3);
			
			foreach ($additionalHours as $hour) {
				// Convert stdClass to array if needed
				if (is_object($hour)) {
					$hour = json_decode(json_encode($hour), true);
				}
				
				// Check if this entry is published
				if (!isset($hour['published']) || $hour['published'] != '1') {
					continue;
				}
				
				// Check if required fields exist
				if (!isset($hour['additional_opening_hours_datefrom']) || !isset($hour['additional_opening_hours_dateto'])) {
					continue;
				}
				
				try {
					// Create dates and handle timezone issues - Joomla calendar field can have timezone offset issues
					// Parse the date and explicitly set it to the user's timezone at noon to avoid day shifts
					$dateFromOriginal = DateTime::createFromFormat('Y-m-d', $hour['additional_opening_hours_datefrom']);
					if (!$dateFromOriginal) {
						// Fallback if format doesn't match
						$dateFromOriginal = new DateTime($hour['additional_opening_hours_datefrom']);
					}
					$dateFromOriginal->setTime(12, 0, 0); // Set to noon to avoid timezone day shifts

					$dateToOriginal = DateTime::createFromFormat('Y-m-d', $hour['additional_opening_hours_dateto']);
					if (!$dateToOriginal) {
						// Fallback if format doesn't match
						$dateToOriginal = new DateTime($hour['additional_opening_hours_dateto']);
					}
					$dateToOriginal->setTime(12, 0, 0);
										
					// Create separate dates for comparison (with time set for proper comparison)
					$dateFromCompare = clone $dateFromOriginal;
					$dateToCompare = clone $dateToOriginal;
					$dateFromCompare->setTime(0, 0, 0);
					$dateToCompare->setTime(23, 59, 59);
					
					// Ensure dateTo is not before dateFrom
					if($dateToCompare < $dateFromCompare) {
						$dateToCompare = clone $dateFromCompare;
						$dateToCompare->setTime(23, 59, 59);
						$dateToOriginal = clone $dateFromOriginal;
					}
				} catch (Exception $e) {
					// Skip invalid dates
					continue;
				}
				
				// If show_future is enabled, only show dates that haven't ended yet
				if ($showFuture && $dateToCompare < $currentDate) {
					continue;
				}
				
				$result[] = array(
					'date_from' => $dateFromOriginal,
					'date_to' => $dateToOriginal,
					'open_or_closed' => isset($hour['additional_opening_hours_open_or_closed']) ? $hour['additional_opening_hours_open_or_closed'] : 'closed',
					'fullday' => isset($hour['additional_opening_hours_fullday']) ? $hour['additional_opening_hours_fullday'] : '0',
					'start_time' => isset($hour['additional_opening_hours_starttime']) ? $hour['additional_opening_hours_starttime'] : '',
					'end_time' => isset($hour['additional_opening_hours_endtime']) ? $hour['additional_opening_hours_endtime'] : '',
					'reason' => isset($hour['additional_opening_hours_reason']) ? $hour['additional_opening_hours_reason'] : ''
				);
			}
			
			// Sort by date
			usort($result, function($a, $b) {
				return $a['date_from'] <=> $b['date_from'];
			});
			
			// Limit to the number specified
			if ($showFuture && count($result) > $numberToShow) {
				$result = array_slice($result, 0, $numberToShow);
			}
		}
		
		return $result;
	}

	// Function to get HTML for additional opening hours
	public function getAdditionalOpeningHoursHTML() {
		$additionalHours = $this->getAdditionalOpeningHours();
		$htmlCode = '';
		
		if (!empty($additionalHours)) {
			$htmlCode .= '<div class="scs-additional-hours">';
			$htmlCode .= '<h4>' . Text::_('MOD_SCS_OPENINGHOURS_ADDITIONAL_HOURS_LABEL') . '</h4>';

			// Information for debugging
			// $htmlCode .= '<div>DEBUG:<br/>' . json_encode($additionalHours) . '</div>';
			
			foreach ($additionalHours as $hour) {
				$htmlCode .= '<div class="scs-additional-hour-item">';
				
				// Date range
				$dateFromFormatted = $hour['date_from']->format('d-m-Y');
				$dateToFormatted = $hour['date_to']->format('d-m-Y');
				
				if ($dateFromFormatted === $dateToFormatted) {
					$htmlCode .= '<div class="scs-additional-date">' . $dateFromFormatted . '</div>';
				} else {
					$htmlCode .= '<div class="scs-additional-date">' . $dateFromFormatted . ' - ' . $dateToFormatted . '</div>';
				}
				
				// Status and times
				if ($hour['open_or_closed'] === 'closed') {
					$htmlCode .= '<div class="scs-additional-status scs-closed">' . $this->params->get('ClosedText', 'CLOSED') . '</div>';
				} else {
					$htmlCode .= '<div class="scs-additional-status scs-open">';
					// if ($hour['fullday'] === '1' && $dateFromFormatted === $dateToFormatted) {
					if ($hour['fullday'] === '1') {
						$htmlCode .= $this->params->get('OpenText', 'OPEN') . ' (' . Text::_('MOD_SCS_OPENINGHOURS_FULLDAY_TEXT') . ')';
					} else {
						$htmlCode .= $this->params->get('OpenText', 'OPEN') . ' (' . $hour['start_time'] . ' - ' . $hour['end_time'] . ')';
					}
					$htmlCode .= '</div>';
				}
				
				// Reason
				if (!empty($hour['reason'])) {
					$htmlCode .= '<div class="scs-additional-reason">' . htmlspecialchars($hour['reason']) . '</div>';
				}
				
				$htmlCode .= '</div>';
			}
			
			$htmlCode .= '</div>';
		}
		
		return $htmlCode;
	}
}
